<?php

/**
 * \file Extractions.class.inc
 * \class Extractions
 *
 * \author WAB <support.wab@veremes.com>.
 *
 * 	\brief This file contains the Extractions php class
 *
 * This class defines Rest Api to Vitis Extractions
 * 
 */
require_once __DIR__ . '/ExtractionExtraction.class.inc';
require_once 'Extraction.class.inc';
require_once(dirname($_SERVER['SCRIPT_FILENAME']) . '/class/wab_lib/BusinessObject.class.inc');

class Extractions extends ExtractionExtraction {
    /**
     * @SWG\Definition(
     *   definition="/extractions",
     *   allOf={
     *     @SWG\Schema(ref="#/definitions/extractions")
     *   }
     * )
     * * @SWG\Tag(
     *   name="Extractions",
     *   description="Operations about Extractions"
     * )
     */

    /**
     * construct
     * @param type $aPath url of the request
     * @param type $aValues parameters of the request
     * @param type $properties properties
     */
    function __construct($aPath, $aValues, $properties, $bShortcut = false) {
        parent::__construct($aPath, $aValues, $properties, $bShortcut);
    }

    /**
     * @SWG\Get(path="/extractions",
     *   tags={"Extractions"},
     *   summary="Get Extractions",
     *   description="Request to get Extractions",
     *   operationId="GET",
     *   produces={"application/xml", "application/json", "application/x-vm-json"},
     *   @SWG\Parameter(
     *     name="token",
     *     in="query",
     *     description="user token",
     *     required=true,
     *     type="string"
     *   ),
     * @SWG\Parameter(
     *     name="order_by",
     *     in="query",
     *     description="list of ordering fields",
     *     required=false,
     *     type="string"
     *   ),
     * @SWG\Parameter(
     *     name="sort_order",
     *     in="query",
     *     description="sort_order",
     *     required=false,
     *     type="string"
     *   ),
     * @SWG\Parameter(
     *     name="limit",
     *     in="query",
     *     description="number of element",
     *     required=false,
     *     type="integer",
     *     format="int32"
     *   ),
     * @SWG\Parameter(
     *     name="offset",
     *     in="query",
     *     description="index of first element",
     *     required=false,
     *     type="string",
     *     format="int32"
     *   ),
     * @SWG\Parameter(
     *     name="attributs",
     *     in="query",
     *     description="list of attributs",
     *     required=false,
     *     type="string"
     *   ),
     * @SWG\Parameter(
     *     name="filter",
     *     in="query",
     *     description="filter results",
     *     required=false,
     *     type="string"
     *   ),
     * @SWG\Parameter(
     *     name="distinct",
     *     in="query",
     *     description="delete duplicates",
     *     required=false,
     *     type="boolean"
     *   ),
     *   @SWG\Response(
     *         response=200,
     *         description="extraction Response",
     *         @SWG\Schema(ref="#/definitions/extractions")
     *     )
     *  )
     */
    /**
     * @SWG\Get(path="/extractions/Layers",
     *   tags={"Extractions"},
     *   summary="Get extraction layers",
     *   description="Request to get the layers of the extractions",
     *   operationId="GET",
     *   produces={"application/xml", "application/json", "application/x-vm-json"},
     *   @SWG\Parameter(
     *     name="token",
     *     in="query",
     *     description="user token",
     *     required=true,
     *     type="string"
     *   ),
     *   @SWG\Response(
     *         response=200,
     *         description="Poprerties Response",
     *         @SWG\Schema(ref="#/definitions/extractions")
     *     )
     *  )
     */

    /**
     * get Extractions
     * @return  Extractions
     */
    function GET() {
        if (!empty($this->aPath[2]) && $this->aPath[2] == "Layers")
            $sMessage = $this->getLayers();
        else {
            $aReturn = $this->genericGet($this->aProperties['schema_extraction'], "v_extraction", "bo_id");
            $sMessage = $aReturn['sMessage'];
        }
        return $sMessage;
    }

    /**
     * @SWG\Post(path="/extractions",
     *   tags={"Extractions"},
     *   summary="Add extraction",
     *   description="Request to add Extractions",
     *   operationId="POST",
     *   produces={"application/xml", "application/json"},
     *   @SWG\Parameter(
     *     name="token",
     *     in="query",
     *     description="user token",
     *     required=true,
     *     type="string"
     *   ),
     *   @SWG\Response(
     *         response=200,
     *         description="extraction Response",
     *         @SWG\Schema(ref="#/definitions/extractions")
     *     )
     *  )
     */

    /**
     * insert extraction
     * @return id of the extraction created
     */
    function POST() {
        $aReturn = $this->genericPost($this->aProperties['schema_extraction'], 'extraction', $this->aProperties['schema_extraction'].'.bo_id_extraction_seq', 'bo_id');
        $this->aValues["bo_id"] = $this->aValues['my_vitis_id'];
	eval('$this->' . $this->aValues['tr_status_method'] . '();');
        return $aReturn['sMessage'];
    }

    /**
     * @SWG\Put(path="/extractions/{id_extraction}",
     *   tags={"Extractions"},
     *   summary="update Extractions",
     *   description="Request to update Extractions",
     *   operationId="PUT",
     *   produces={"application/xml", "application/json"},
     *   @SWG\Parameter(
     *     name="token",
     *     in="query",
     *     description="Extraction token",
     *     required=true,
     *     type="string"
     *   ),
     * * @SWG\Parameter(
     *     name="id_extraction",
     *     in="path",
     *     description="id of the Extractions",
     *     required=true,
     *     type="integer",
     *     format = "int32"
     *   ),
     * @SWG\Response(
     *         response=200,
     *         description="Poprerties Response",
     *         @SWG\Schema(ref="#/definitions/extractions")
     *     )
     *  )
     */

    /**
     * update extractions
     * @return id of extractions updated or error object if a extractions is not updated
     */
    function PUT() {
        // Upload l'acte d'engagement.
        $this->uploadDocument('ae', 'extraction_extraction_extraction');
        //
        $aReturn = $this->genericPut($this->aProperties['schema_extraction'], 'extraction', 'bo_id');
        $this->aValues["bo_id"] = $this->aValues['my_vitis_id'];
        // Méthode de transition.
        if (!empty($this->aValues['tr_status_method']))
            eval('$this->' . $this->aValues['tr_status_method'] . '();');
        return $aReturn['sMessage'];
    }

    /**
     * @SWG\Delete(path="/extractions",
     *   tags={"Extractions"},
     *   summary="delete Extractions",
     *   description="Request to delete Extractions",
     *   operationId="DELETE",
     *   produces={"application/xml", "application/json"},
     *   @SWG\Parameter(
     *     name="token",
     *     in="query",
     *     description="token",
     *     required=true,
     *     type="string"
     *   ),
     * * @SWG\Parameter(
     *     name="idList",
     *     in="query",
     *     description="id of the extraction",
     *     required=true,
     *     type="string"
     *   ),
     * @SWG\Response(
     *         response=200,
     *         description="extraction Response",
     *         @SWG\Schema(ref="#/definitions/extractions")
     *     )
     *  )
     */
    /**
     * @SWG\Delete(path="/extractions/{id_extraction}",
     *   tags={"Extractions"},
     *   summary="delete Extractions",
     *   description="Request to delete Extractions",
     *   operationId="DELETE",
     *   produces={"application/xml", "application/json"},
     *   @SWG\Parameter(
     *     name="token",
     *     in="query",
     *     description="Extraction token",
     *     required=true,
     *     type="string"
     *   ),
     * * @SWG\Parameter(
     *     name="id_extraction",
     *     in="path",
     *     description="id of the Extractions",
     *     required=true,
     *     type="integer",
     *     format = "int32"
     *   ),
     * @SWG\Response(
     *         response=200,
     *         description="Poprerties Response",
     *         @SWG\Schema(ref="#/definitions/extractions")
     *     )
     *  )
     */

    /**
     * delete extractions
     * @return id of extractions deleted or error object if a extractions is not deleted
     */
    function DELETE() {
        $aReturn = $this->genericDelete($this->aProperties['schema_extraction'], 'extraction', 'bo_id');
        return $aReturn['sMessage'];
    }

    
    /**
    * Transition Method trGenerer
    */
    function trGenerer() {
        $this->oBO = new WabBusinessObject($this->oConnection->oBd, array(), $this->aValues, $this->aProperties, 'extraction', $this->aValues['bo_id']);
        // Check si la table existe si oui change l'état 
        $this->oBO->wabSetStatus($this->aProperties["schema_extraction"], "extraction", "bo_id", $this->aValues["bo_id"], "AE Généré");
        
    }
		
    /**
    * Transition Method trNouveau
    */
    function trNouveau() {
        $this->oBO = new WabBusinessObject($this->oConnection->oBd, array(), $this->aValues, $this->aProperties, 'extraction', $this->aValues['bo_id']);
        
        // Check si la table existe si oui change l'état 
        $this->oBO->wabSetStatus($this->aProperties["schema_extraction"], "extraction", "bo_id", $this->aValues["bo_id"], "Nouveau");
        
    }
		
    /**
    * Transition Method trTerminer
    */
    function trTerminer() {
        $this->oBO = new WabBusinessObject($this->oConnection->oBd, array(), $this->aValues, $this->aProperties, 'extraction', $this->aValues['bo_id']);
        
        // Check si la table existe si oui change l'état 
        $this->oBO->wabSetStatus($this->aProperties["schema_extraction"], "extraction", "bo_id", $this->aValues["bo_id"], "Terminée");
        
    }
		
    /**
    * Transition Method trAutoriser
    */
    function trAutoriser() {
        $this->oBO = new WabBusinessObject($this->oConnection->oBd, array(), $this->aValues, $this->aProperties, 'extraction', $this->aValues['bo_id']);
        $this->oBO->insertOrder('D16ECS75P', 'bo_id='.$this->aValues['bo_id'], $this->aValues['priority_id'], 2 );
        // Check si la table existe si oui change l'état 
        $this->oBO->wabSetStatus($this->aProperties["schema_extraction"], "extraction", "bo_id", $this->aValues["bo_id"], "Autorisée");
        
    }
		
    /**
    * Transition Method trDemander
    */
    function trDemander() {
        $this->oBO = new WabBusinessObject($this->oConnection->oBd, array(), $this->aValues, $this->aProperties, 'extraction', $this->aValues['bo_id']);
        $this->oBO->insertOrder('D14RHVRJC', 'bo_id='.$this->aValues['bo_id'], 1, 2);
        // Check si la table existe si oui change l'état 
        $this->oBO->wabSetStatus($this->aProperties["schema_extraction"], "extraction", "bo_id", $this->aValues["bo_id"], "AE Demandé");
        
    }
		
    /**
    * Transition Method trResoumettre
    */
    function trResoumettre() {
        $this->oBO = new WabBusinessObject($this->oConnection->oBd, array(), $this->aValues, $this->aProperties, 'extraction', $this->aValues['bo_id']);
        
        // Check si la table existe si oui change l'état 
        $this->oBO->wabSetStatus($this->aProperties["schema_extraction"], "extraction", "bo_id", $this->aValues["bo_id"], "AE Généré");
        //$this->sendmail(9904);
    }
		
    /**
    * Transition Method trSoumettre
    */
    function trSoumettre() {
        $this->oBO = new WabBusinessObject($this->oConnection->oBd, array(), $this->aValues, $this->aProperties, 'extraction', $this->aValues['bo_id']);
        
        // Check si la table existe si oui change l'état 
        $this->oBO->wabSetStatus($this->aProperties["schema_extraction"], "extraction", "bo_id", $this->aValues["bo_id"], "AE Soumis");
        $this->oBO->sendmail(39);
    }
		
    /**
    * Transition Method trRefuser
    */
    function trRefuser() {
        $this->oBO = new WabBusinessObject($this->oConnection->oBd, array(), $this->aValues, $this->aProperties, 'extraction', $this->aValues['bo_id']);
        
        // Check si la table existe si oui change l'état 
        $this->oBO->wabSetStatus($this->aProperties["schema_extraction"], "extraction", "bo_id", $this->aValues["bo_id"], "Refusée");
        //$this->sendmail(9902);
    }
    
    /**
     * Liste des couches.
     */
    function getLayers() {
        require $this->sRessourcesFile;
        // Liste des groupes de l'utilisateur.
        $sSql = str_replace('[schema_framework]', $this->aProperties['schema_framework'], $aSql['getUserGroups']);
        $sSql = str_replace('[user_id]', $_SESSION['ses_user_id'], $sSql);
        $oPDOresult = $this->oConnection->oBd->executeWithParams($sSql, array());
        if (!$this->oConnection->oBd->enErreur()) {
            $aResult = $this->oConnection->oBd->getResultTableAssoc($oPDOresult);
            foreach($aResult as $aUserGroup) {
                $aUserGroups[] = $aUserGroup['name'];
            }
            // Restriction de l'utilisateur suivant ses groupes.
            if (in_array('interne', $aUserGroups))
                $sSqlParams = '';
            else if (in_array('restreint', $aUserGroups))
                $sSqlParams = " WHERE restriction <> 'interne'";
            else
                $sSqlParams = " WHERE restriction = 'public'";
            // Liste des cartes.
            $this->aFields['treeview_layers'] = array();
            $sSql = str_replace('[schema_extraction]', $this->aProperties['schema_extraction'], $aSql['getRtLayer']);
            $sSql = str_replace('[sql_params]', $sSqlParams, $sSql);
            $oPDOresult = $this->oConnection->oBd->executeWithParams($sSql, array());
            if (!$this->oConnection->oBd->enErreur()) {
                $aRtLayer = $this->oConnection->oBd->getResultTableAssoc($oPDOresult);
                $aLayers = array();
                if (!empty($aRtLayer)) {
                    foreach($aRtLayer as $aLayer)
                        $aLayers[$aLayer['category']][] = array("text" => $aLayer['layer'], "value" => $aLayer['layer_id']);
                }
                $this->aFields['layers'] = $aLayers;
                $aXmlRacineAttribute['status']=1;
                $sMessage = $this->asDocument('','vitis',$this->aValues['sEncoding'],True,$aXmlRacineAttribute,$this->aValues['sSourceEncoding'],$this->aValues['output']);
            }
            else {
                $aXmlRacineAttribute['status']=0;
                $this->aFields['error'] = "Request error.";
                $sMessage = $this->asDocument('','vitis',$this->aValues['sEncoding'],True,$aXmlRacineAttribute,$this->aValues['sSourceEncoding'],$this->aValues['output']);
            }
        }
        else {
            $aXmlRacineAttribute['status']=0;
            $this->aFields['error'] = "Request error.";
            $sMessage = $this->asDocument('','vitis',$this->aValues['sEncoding'],True,$aXmlRacineAttribute,$this->aValues['sSourceEncoding'],$this->aValues['output']);
        }
        return $sMessage;
    }
}
?>